(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 11.1' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     53832,       1404]
NotebookOptionsPosition[     48844,       1275]
NotebookOutlinePosition[     49201,       1291]
CellTagsIndexPosition[     49158,       1288]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["2-d classical Ising Monte Carlo simulator", "Title",
 CellChangeTimes->{{3.74651637220112*^9, 3.746516384345118*^9}, {
  3.746956260108138*^9, 
  3.746956261724502*^9}},ExpressionUUID->"9b3c0e5a-c573-4b61-a303-\
a50bac06aac4"],

Cell[TextData[{
 "Ising model is the go-to toy model of a system undergoing a phase \
transition to become ferromagnetic. The lattice consists of spins with values \
either -1 or 1. They have nearest neighbour (i.e. directly up, down, left or \
right in 2d) interactions and they can also be coupled to an external \
magnetic field. The Hamiltonian is -J ",
 Cell[BoxData[
  SubscriptBox["\[CapitalSigma]", 
   RowBox[{
    RowBox[{"<", "i"}], ",", 
    RowBox[{"j", ">"}]}]]], "Input",
  CellChangeTimes->{{3.746955760259287*^9, 3.7469557739049587`*^9}},
  ExpressionUUID->"31aa2577-fd1c-4277-929d-e154cf222b1c"],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Sigma]", "i"], 
    SubscriptBox["\[Sigma]", "j"]}], TraditionalForm]],ExpressionUUID->
  "de295261-700f-4ce5-91c0-2e7f3ff76bb8"],
 "-h ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[CapitalSigma]", "i"], " ", 
   SubscriptBox["\[Sigma]", "i"]}]], "Input",
  CellChangeTimes->{{3.746955760259287*^9, 3.746955808529867*^9}},
  ExpressionUUID->"2082c427-f5cd-4c7d-9ba8-7dabc39da407"],
 ", where ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Sigma]", "i"], TraditionalForm]],ExpressionUUID->
  "0899e5f5-78b4-4aa4-be0c-0e388162f47c"],
 " is the spin at site i, <i,j> indicates a sum over the nearest neighbour \
pairs, J is a coupling constant (usually J = 1) and h is the value of the \
external magnetic field. Low-energy states tend to have their spins aligned \
while in high-energy states the spin changes sign from site to site.\n\nThe \
typical thing to do would be to compute the canonical partition function, \
Z[h] = ",
 Cell[BoxData[
  SubscriptBox["\[CapitalSigma]", "configurations"]], "Input",
  CellChangeTimes->{{3.746955935751615*^9, 3.746955945556892*^9}},
  ExpressionUUID->"8eb48716-183d-4461-86b1-da0d32bab7aa"],
 " Exp[-\[Beta] H[h]] and to study many properties of the system including \
magnetization. However, this only doable for a few cases and we have to \
resort to numerical magic. The 2d model is integrable when h=0 but we also \
wish to consider the effect of the magnetic field. In two and higher \
dimensions, the system undergoes a 2nd order phase transition from ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[LeftAngleBracket]", 
     SubscriptBox["\[Sigma]", "i"], "\[RightAngleBracket]"}], "=", "0"}], 
   TraditionalForm]],ExpressionUUID->"5ab1dd50-aafb-4c08-8149-57dc32dc2f04"],
 " (paramagnetic) phase to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[LeftAngleBracket]", 
     SubscriptBox["\[Sigma]", "i"], "\[RightAngleBracket]"}], "\[NotEqual]", 
    "0"}], TraditionalForm]],ExpressionUUID->
  "a07e1155-fc7f-498f-94bd-1784dcd0fea1"],
 " (ferromagnetic) phase at a special value of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Beta]", "=", 
    SubscriptBox["\[Beta]", "c"]}], TraditionalForm]],ExpressionUUID->
  "caeacf8b-ba43-4559-a3da-c6a8e3335647"],
 " and at h=0.\n\nWe wish to study this system and its phase transition on a \
two-dimensional periodic square lattice. We do this using Monte Carlo \
simulation. We construct both a local and a cluster update method. The idea \
of Monte Carlo simulations is to generate typical configurations of the \
systems by using incremental updates to the configuration that are accepted \
or rejected with some \[Beta] dependent probability. It can be shown that the \
statistical properties of these generated states approach the thermodynamic \
values."
}], "Text",
 CellChangeTimes->{{3.7469556050454884`*^9, 3.7469564187790337`*^9}, {
  3.7469614508983593`*^9, 3.746961496994623*^9}, {3.74696557181209*^9, 
  3.746965576973339*^9}},ExpressionUUID->"b7040563-df6a-43ce-8104-\
d86234ae13d2"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Setup", "Chapter"]], "Section",
 CellChangeTimes->{{3.746515796673381*^9, 
  3.7465157992890053`*^9}},ExpressionUUID->"99eafa64-8388-4295-ba90-\
f9acbafb29be"],

Cell[CellGroupData[{

Cell["Generic functions:", "Subsection",
 CellChangeTimes->{{3.746956449666028*^9, 
  3.7469564526495247`*^9}},ExpressionUUID->"18ec621e-585d-4060-8e7d-\
6d2be6ea8cc1"],

Cell[CellGroupData[{

Cell["0. For good random numbers, initialize Mersenne Twister", \
"Subsubsection",
 CellChangeTimes->{{3.746956514930093*^9, 
  3.7469565373706093`*^9}},ExpressionUUID->"61df1d94-7c8f-4b08-88b5-\
80d41d496fba"],

Cell[BoxData[
 RowBox[{"SeedRandom", "[", 
  RowBox[{"Method", "\[Rule]", " ", "\"\<MersenneTwister\>\""}], 
  "]"}]], "Input",ExpressionUUID->"93daf99f-b2fa-4394-ad5f-7aa3fe99cec2"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
1. Construct a square lattice (array) of size L^2. Give it some initial \
state, i.e. fill it with -1\[CloseCurlyQuote]s and 1\[CloseCurlyQuote]s. \
Stick to L above 16 but below 200.\
\>", "Subsubsection",
 CellChangeTimes->{{3.746956454880382*^9, 3.7469564887294416`*^9}, {
  3.746956572723748*^9, 3.746956574547825*^9}, {3.746962864188995*^9, 
  3.746962887110404*^9}},ExpressionUUID->"2c104146-3087-412a-aea7-\
a19b313f60c6"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rlattice", "[", 
   RowBox[{"Lx_", ",", "Ly_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"RandomInteger", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Lx", ",", "Ly"}], "}"}]}], "]"}], "*", "2"}], "-", 
   "1"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"lattice0", "=", 
    RowBox[{"rlattice", "[", 
     RowBox[{"80", ",", "80"}], "]"}]}], ";"}], " "}]}], "Input",
 CellChangeTimes->{{3.745564132754978*^9, 3.745564134735841*^9}, {
   3.74556436247572*^9, 3.745564440467956*^9}, {3.745564591351396*^9, 
   3.7455646034727917`*^9}, 3.745565275691599*^9, 3.745577994707058*^9, {
   3.7455895559080887`*^9, 3.745589555979636*^9}, {3.745591453883438*^9, 
   3.7455914539833384`*^9}, {3.745592026663048*^9, 3.745592028251718*^9}, {
   3.746516082684083*^9, 3.746516095043969*^9}, {3.746962951252761*^9, 
   3.746962994968617*^9}, {3.746963495082952*^9, 3.746963496544736*^9}, {
   3.746963583905953*^9, 3.7469635866419086`*^9}, {3.7469637349552813`*^9, 
   3.746963736652268*^9}, 3.746963907883634*^9, {3.74990490337571*^9, 
   3.749904905623494*^9}},ExpressionUUID->"9c39febb-a112-4f1e-b2cc-\
894fef987861"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
2. Make the lattice periodic, i.e. construct functions s.t.  \[Sigma](L+1,y) \
= \[Sigma](1,y), \[Sigma](x,1+L)=\[Sigma](x,1), \[Sigma](1-1,y)=\[Sigma](0,y) \
etc.\
\>", "Subsubsection",
 CellChangeTimes->{{3.7469566012521067`*^9, 
  3.746956681117943*^9}},ExpressionUUID->"3f441c09-93ab-43ad-8733-\
54f165d749b9"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"coordfix", "[", 
     RowBox[{"n_", ",", "L_"}], "]"}], ":=", 
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{
       RowBox[{"n", "-", "1"}], ",", "L"}], "]"}], "+", "1"}]}], ";"}], " ", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"L", "+", 
     RowBox[{"1", "~", "1"}]}], ",", " ", 
    RowBox[{"0", "~", "L"}]}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"siteup", "[", "site_", "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"coordfix", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"site", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "+", "1"}], ",", 
       RowBox[{
        RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ",", 
     RowBox[{"site", "[", 
      RowBox[{"[", "2", "]"}], "]"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"sitedown", "[", "site_", "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"coordfix", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"site", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "-", "1"}], ",", 
       RowBox[{
        RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ",", 
     RowBox[{"site", "[", 
      RowBox[{"[", "2", "]"}], "]"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"siteright", "[", "site_", "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"site", "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"coordfix", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"site", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "+", "1"}], ",", 
       RowBox[{
        RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"siteleft", "[", "site_", "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"site", "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"coordfix", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"site", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "-", "1"}], ",", 
       RowBox[{
        RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.74558632283622*^9, 3.745586469884533*^9}, {
  3.749903944535172*^9, 3.7499039449604*^9}, {3.749904024969264*^9, 
  3.749904041498006*^9}},ExpressionUUID->"49254c77-89c5-48d5-9bcb-\
cb7a7a4eb1a7"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
3. Construct a function that computes the Hamiltonian on the lattice.\
\>", "Subsubsection",
 CellChangeTimes->{{3.7469565627875023`*^9, 
  3.746956599099873*^9}},ExpressionUUID->"fd907b3e-1a13-4c41-9ed7-\
c678a87a4c60"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Ham", "[", 
     RowBox[{"J_", ",", "h_"}], "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "J"}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"lattice0", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "*", 
            RowBox[{"lattice0", "[", 
             RowBox[{"[", 
              RowBox[{"-", "1"}], "]"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{"lattice0", "[", 
             RowBox[{"[", 
              RowBox[{"All", ",", "1"}], "]"}], "]"}], "*", 
            RowBox[{"lattice0", "[", 
             RowBox[{"[", 
              RowBox[{"All", ",", 
               RowBox[{"-", "1"}]}], "]"}], "]"}]}]}], "//", "Total"}], ")"}],
         "+", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"lattice0", "[", 
              RowBox[{"[", 
               RowBox[{"1", ";;", 
                RowBox[{"-", "2"}]}], "]"}], "]"}], "*", 
             RowBox[{"lattice0", "[", 
              RowBox[{"[", 
               RowBox[{"2", ";;", 
                RowBox[{"-", "1"}]}], "]"}], "]"}]}], "//", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Total", "[", 
               RowBox[{"#", ",", "2"}], "]"}], "&"}], ")"}]}], ")"}], "+", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"lattice0", "[", 
              RowBox[{"[", 
               RowBox[{"All", ",", 
                RowBox[{"1", ";;", 
                 RowBox[{"-", "2"}]}]}], "]"}], "]"}], "*", 
             RowBox[{"lattice0", "[", 
              RowBox[{"[", 
               RowBox[{"All", ",", 
                RowBox[{"2", ";;", 
                 RowBox[{"-", "1"}]}]}], "]"}], "]"}]}], "//", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Total", "[", 
               RowBox[{"#", ",", "2"}], "]"}], "&"}], ")"}]}], ")"}]}], 
         ")"}]}], ")"}]}], "-", 
     RowBox[{"h", " ", 
      RowBox[{"Total", "[", 
       RowBox[{"lattice0", ",", "2"}], "]"}]}]}]}], ";"}], " ", 
  RowBox[{"(*", 
   RowBox[{
   "First", " ", "two", " ", "terms", " ", "are", " ", "for", " ", 
    "periodicity", " ", "at", " ", "the", " ", 
    RowBox[{"boundary", ".", " ", "The"}], " ", "two", " ", "afterwards", " ",
     "are", " ", "bulk", " ", 
    RowBox[{"terms", ".", " ", "Last"}], " ", "term", " ", "is", " ", "for", 
    " ", "the", " ", "magnetic", " ", "field"}], "*)"}]}]], "Input",
 CellChangeTimes->{{3.745564665008868*^9, 3.74556466874533*^9}, {
  3.745564980145627*^9, 3.745564980969576*^9}, {3.745565033773905*^9, 
  3.74556503922709*^9}, {3.7455651126488047`*^9, 3.745565181062994*^9}, {
  3.7455652266727467`*^9, 3.745565240984501*^9}, {3.745565292962309*^9, 
  3.7455653408205214`*^9}, {3.74556539162927*^9, 3.745565502515851*^9}, {
  3.745565627978703*^9, 3.74556567571535*^9}, {3.74557014286934*^9, 
  3.74557016169425*^9}, {3.749903963521802*^9, 
  3.749904019369005*^9}},ExpressionUUID->"4096833e-5303-4bd0-a79e-\
ed4ad8913ede"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
4. Construct a function that gives you a random initial site.\
\>", "Subsubsection",
 CellChangeTimes->{{3.746956716294778*^9, 
  3.7469567305112667`*^9}},ExpressionUUID->"f2e72b12-f57c-489a-bdd1-\
2f86c0748720"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"randomsite", "[", "]"}], ":=", 
   RowBox[{"sites", "[", 
    RowBox[{"[", 
     RowBox[{"RandomInteger", "[", 
      RowBox[{"{", 
       RowBox[{"1", ",", 
        RowBox[{"Times", "@@", 
         RowBox[{"Dimensions", "[", "lattice0", "]"}]}]}], "}"}], "]"}], 
     "]"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.745590689943828*^9, 3.7455906982483597`*^9}, {
  3.745590949826017*^9, 3.74559095042035*^9}, {3.745591471101725*^9, 
  3.745591481682548*^9}, {3.746956559603714*^9, 
  3.746956560091663*^9}},ExpressionUUID->"7006cbf9-205b-4cf2-b031-\
c994d43de44c"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Local update", "Subsection",
 CellChangeTimes->{{3.746515805223975*^9, 
  3.746515806223929*^9}},ExpressionUUID->"7f602e24-833f-440f-b2f8-\
757f88ead821"],

Cell["\<\
Local updates are done by making a pre-emptive flip of a single spin. If the \
energy of the system goes down, the spin flip is accepted. If the energy goes \
up, it is accepted with a probability Exp[- \[Beta] \[Delta]H], where \
\[Delta]H is the change of energy and reverted with probability 1-Exp[- \
\[Beta] \[Delta]H]. Note that change occurs in a small area affecting only 4 \
\[OpenCurlyDoubleQuote]bonds\[CloseCurlyDoubleQuote] (due to 4 neighbours) \
and the energy shift due to the magnetic field so you can stick to computing \
a local part of the Hamiltonian to determine the probability of flipping or \
not. This is the so-called Metropolis method.\
\>", "Text",
 CellChangeTimes->{{3.746956805152954*^9, 3.7469569228760967`*^9}, {
  3.746956972310603*^9, 3.746956982607177*^9}, {3.746961340607101*^9, 
  3.746961425975679*^9}, {3.746962178040091*^9, 3.746962180382947*^9}, {
  3.746962226367577*^9, 3.746962267390121*^9}, {3.7469624140352*^9, 
  3.746962420871744*^9}},ExpressionUUID->"55d33a9c-336d-4330-83f7-\
136351ebe0ba"],

Cell["\<\
These spin-by-spin flips are applied to the whole lattice and they are \
flipped separately. The order is not very important. You can do them row by \
row, or doing even and odd sites separately (checker board) or in a \
completely random order.\
\>", "Text",
 CellChangeTimes->{{3.746962473029163*^9, 
  3.7469625893944483`*^9}},ExpressionUUID->"0cecc518-0fae-4b80-852d-\
1d066b1bd577"],

Cell[CellGroupData[{

Cell["\<\
5. Construct a function to determine, whether a single spin should be flipped \
or not.\
\>", "Subsubsection",
 CellChangeTimes->{{3.746962215552417*^9, 3.7469622172103977`*^9}, {
  3.746962401412509*^9, 3.7469624560595207`*^9}, {3.7469634577763443`*^9, 
  3.746963458646936*^9}},ExpressionUUID->"83454930-bae6-4357-809c-\
5b3a00a914c9"],

Cell[TextData[{
 "I calculate the local Hamiltonian at the site and determine whether the \
energy contribution is negative or positive. If it is positive, I flip the \
spin (s.t. energy goes down). If it is negative, I flip the spin with \
probability exp(-2 \[Beta] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["H", "loc"], TraditionalForm]],ExpressionUUID->
  "01a51901-c490-4c3d-873a-dca842ac3664"],
 ") (as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"H", "[", "flipped", "]"}], "-", 
     RowBox[{"H", "[", "original", "]"}]}], "=", 
    RowBox[{"2", 
     SubscriptBox["J\[Sigma]", "i"]}]}], TraditionalForm]],ExpressionUUID->
  "4d891188-fcce-423f-b598-129f1ab37314"],
 "*(neighbour \[Sigma]\[CloseCurlyQuote]s)+2h ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Sigma]", "i"], TraditionalForm]],ExpressionUUID->
  "5ad3bca9-59ea-4b70-98ad-8f20ce6592c2"],
 "."
}], "Text",
 CellChangeTimes->{{3.7469622218023376`*^9, 
  3.74696238617802*^9}},ExpressionUUID->"c34ec28b-1878-4ea6-9bdd-\
d8ecaedadb17"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"localham", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"coordx_", ",", "coordy_"}], "}"}], ",", "J_", ",", "h_"}], 
     "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "J"}], "*", 
      RowBox[{"lattice0", "[", 
       RowBox[{"[", 
        RowBox[{"coordx", ",", "coordy"}], "]"}], "]"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"lattice0", "[", 
         RowBox[{"[", 
          RowBox[{
           RowBox[{"coordfix", "[", 
            RowBox[{
             RowBox[{"coordx", "+", "1"}], ",", 
             RowBox[{
              RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ",", "coordy"}], "]"}],
          "]"}], "+", 
        RowBox[{"lattice0", "[", 
         RowBox[{"[", 
          RowBox[{
           RowBox[{"coordfix", "[", 
            RowBox[{
             RowBox[{"coordx", "-", "1"}], ",", 
             RowBox[{
              RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ",", "coordy"}], "]"}],
          "]"}], "+", 
        RowBox[{"lattice0", "[", 
         RowBox[{"[", 
          RowBox[{"coordx", ",", 
           RowBox[{"coordfix", "[", 
            RowBox[{
             RowBox[{"coordy", "+", "1"}], ",", 
             RowBox[{
              RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "]"}], "]"}], "+", 
        RowBox[{"lattice0", "[", 
         RowBox[{"[", 
          RowBox[{"coordx", ",", 
           RowBox[{"coordfix", "[", 
            RowBox[{
             RowBox[{"coordy", "-", "1"}], ",", 
             RowBox[{
              RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "]"}], "]"}]}], 
       ")"}]}], "-", 
     RowBox[{"h", "*", 
      RowBox[{"lattice0", "[", 
       RowBox[{"[", 
        RowBox[{"coordx", ",", "coordy"}], "]"}], "]"}]}]}]}], ";"}], " ", 
  RowBox[{"(*", 
   RowBox[{"local", " ", "hamiltonian", " ", "term"}], "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"sites", ":=", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"i", ",", "j"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{
          RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "1", ",", 
         RowBox[{
          RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "}"}]}], "]"}], "//", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Flatten", "[", 
        RowBox[{"#", ",", "1"}], "]"}], "&"}], ")"}]}]}], ";"}], 
  RowBox[{"(*", 
   RowBox[{
   "all", " ", "the", " ", "sites", " ", "on", " ", "the", " ", "lattice"}], 
   "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"metropolisflip", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"coordx_", ",", "coordy_"}], "}"}], ",", "J_", ",", "h_", ",", 
      "\[Beta]_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "localenergy", "}"}], ",", 
      RowBox[{
       RowBox[{"localenergy", " ", "=", " ", 
        RowBox[{"localham", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"coordx", ",", "coordy"}], "}"}], ",", "J", ",", "h"}], 
         "]"}]}], ";", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"localenergy", ">", "0"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"lattice0", "[", 
            RowBox[{"[", 
             RowBox[{"coordx", ",", "coordy"}], "]"}], "]"}], "=", 
           RowBox[{"-", 
            RowBox[{"lattice0", "[", 
             RowBox[{"[", 
              RowBox[{"coordx", ",", "coordy"}], "]"}], "]"}]}]}], ";"}], ",", 
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"RandomReal", "[", 
              RowBox[{"{", 
               RowBox[{"0", ",", "1"}], "}"}], "]"}], "<", 
             RowBox[{"Exp", "[", 
              RowBox[{"2", "\[Beta]", " ", "localenergy"}], "]"}]}], ",", 
            RowBox[{
             RowBox[{"lattice0", "[", 
              RowBox[{"[", 
               RowBox[{"coordx", ",", "coordy"}], "]"}], "]"}], "=", 
             RowBox[{"-", 
              RowBox[{"lattice0", "[", 
               RowBox[{"[", 
                RowBox[{"coordx", ",", "coordy"}], "]"}], "]"}]}]}], ",", 
            RowBox[{"1", ";"}]}], "]"}], ";"}]}], "]"}]}]}], "]"}]}], ";"}], 
  " ", 
  RowBox[{"(*", 
   RowBox[{
   "Look", " ", "at", " ", "local", " ", "hamiltonian", " ", "change", " ", 
    "for", " ", "an", " ", "individual", " ", "spin", " ", "flip"}], 
   "*)"}]}]}], "Input",
 CellChangeTimes->{{3.745565585706464*^9, 3.745565620537574*^9}, {
   3.745565778189782*^9, 3.7455659914813337`*^9}, {3.7455661161562357`*^9, 
   3.745566134068777*^9}, {3.745566167981533*^9, 3.745566234400988*^9}, {
   3.745566313417444*^9, 3.745566337513446*^9}, {3.745566370306726*^9, 
   3.7455665059233*^9}, {3.7455665370146723`*^9, 3.7455665374305696`*^9}, {
   3.745568057071156*^9, 3.7455680747594643`*^9}, {3.745569240519659*^9, 
   3.745569260575803*^9}, {3.7455695956156597`*^9, 3.745569596434795*^9}, {
   3.745570180958358*^9, 3.7455702012151337`*^9}, 3.745578002895666*^9, 
   3.746962424634762*^9, {3.749903941192449*^9, 3.749903941551927*^9}, {
   3.749904064099622*^9, 
   3.749904130811833*^9}},ExpressionUUID->"56115e78-f935-427c-8548-\
79261f708911"]
}, Open  ]],

Cell[CellGroupData[{

Cell["6. Construct a function for a lattice-wide update.", "Subsubsection",
 CellChangeTimes->{{3.746962429290292*^9, 3.746962470392929*^9}, {
  3.746963462551528*^9, 
  3.746963462793186*^9}},ExpressionUUID->"8f4a53c4-3da9-4248-ab55-\
eba7508aa5e0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"metropolisupdate", "[", 
   RowBox[{"J_", ",", "h_", ",", "\[Beta]_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"sites0", "=", 
      RowBox[{"RandomSample", "[", "sites", "]"}]}], "}"}], ",", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"metropolisflip", "[", 
          RowBox[{"#", ",", "J", ",", "h", ",", "\[Beta]"}], "]"}], "&"}], 
        ")"}], ",", "sites0"}], "]"}], ";"}]}], "]"}], " ", 
  RowBox[{"(*", 
   RowBox[{
   "Do", " ", "the", " ", "local", " ", "updates", " ", "in", " ", "some", 
    " ", "random", " ", "order"}], "*)"}]}]], "Input",
 CellChangeTimes->{{3.745565585706464*^9, 3.745565620537574*^9}, {
   3.745565778189782*^9, 3.7455659914813337`*^9}, {3.7455661161562357`*^9, 
   3.745566134068777*^9}, {3.745566167981533*^9, 3.745566234400988*^9}, {
   3.745566313417444*^9, 3.745566337513446*^9}, {3.745566370306726*^9, 
   3.7455665059233*^9}, {3.7455665370146723`*^9, 3.7455665374305696`*^9}, {
   3.745568057071156*^9, 3.7455680747594643`*^9}, {3.745569240519659*^9, 
   3.745569260575803*^9}, {3.7455695956156597`*^9, 3.745569596434795*^9}, {
   3.745570180958358*^9, 3.7455702012151337`*^9}, 3.745578002895666*^9, 
   3.746962424634762*^9, {3.749904136923233*^9, 
   3.749904144124235*^9}},ExpressionUUID->"56115e78-f935-427c-8548-\
79261f708911"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "7. Plot the lattice using MatrixPlot with Dynamic updating and run a few \
thousand rounds (for small lattices, for bigger lattices you need much less) \
of your update method. Track the total energy of the system and the \
magnetization: M= ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", 
    RowBox[{"L", "^", "2"}]], TraditionalForm]],ExpressionUUID->
  "817106d1-785a-493a-945c-b16c6ace51d2"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[CapitalSigma]", "i"], 
    SubscriptBox["\[Sigma]", "i"]}], TraditionalForm]],ExpressionUUID->
  "0a928bb9-47fd-4428-bd0e-161500db6df4"],
 ". Keeping h=0 and J = 1, when does the system exhibit spontaneous \
magnetization? (It might be better to track |M|). The analytic value is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", "/", "2"}], 
    RowBox[{"Log", "[", 
     RowBox[{"1", "+", 
      SqrtBox["2"]}], "]"}]}], TraditionalForm]],ExpressionUUID->
  "089dec50-abec-44de-a257-75ce3930c6cb"],
 "=0.44... (Note: Beyond the critical value ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Beta]", "c"], TraditionalForm]],ExpressionUUID->
  "11c20784-653f-4d08-8ff0-162a03ed206f"],
 ", the magnetization would not go to zero even when considering larger \
lattices) Analysis is skipped in these solutions"
}], "Subsubsection",
 CellChangeTimes->{{3.746962599410112*^9, 3.746962837755047*^9}, {
  3.746963426313212*^9, 3.746963465751136*^9}, {3.746963614856179*^9, 
  3.746963616534127*^9}, {3.746963861613131*^9, 3.746963870970484*^9}, {
  3.746964090989456*^9, 3.7469640921994267`*^9}, {3.749904981106173*^9, 
  3.7499049813792257`*^9}},ExpressionUUID->"8f17aa57-d4a5-45da-8acb-\
6c1d68fd666c"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"magnlist", " ", "=", " ", 
   RowBox[{"{", "}"}]}], ";", 
  RowBox[{"enlist", "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"metropolisupdate", "[", 
     RowBox[{"1", ",", "0", ",", "0.42"}], "]"}], ";", 
    RowBox[{"magnlist", "=", 
     RowBox[{"Append", "[", 
      RowBox[{"magnlist", ",", 
       RowBox[{"Mean", "[", 
        RowBox[{"Mean", "[", 
         RowBox[{"lattice0", "//", "N"}], "]"}], "]"}]}], "]"}]}], ";", 
    RowBox[{"enlist", "=", 
     RowBox[{"Append", "[", 
      RowBox[{"enlist", ",", 
       RowBox[{"Ham", "[", 
        RowBox[{"1", ",", "0"}], "]"}]}], "]"}]}], ";"}], ",", 
   RowBox[{"{", "500", "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.746962939664157*^9, 3.7469629407901497`*^9}, {
   3.746962972285514*^9, 3.746963041153617*^9}, {3.746963085251107*^9, 
   3.746963160898024*^9}, 3.746963297857852*^9, 3.746963328525338*^9, {
   3.746963360002059*^9, 3.746963389915484*^9}, {3.746963474241054*^9, 
   3.746963504242919*^9}, 3.746963541599162*^9, 
   3.746963640328237*^9},ExpressionUUID->"d8b87073-f51b-4329-81f0-\
7694664750f3"],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.746963489078781*^9, 3.746963541086191*^9, 
  3.746963578789949*^9, 3.7469636363584538`*^9, 3.746963726075571*^9, 
  3.746963884367854*^9, 
  3.7499044080762243`*^9},ExpressionUUID->"d6a1620d-bdbd-402a-8cd4-\
c79fde3f63e0"]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"MatrixPlot", "[", "lattice0", "]"}], ",", 
    RowBox[{"ListPlot", "[", 
     RowBox[{"magnlist", ",", 
      RowBox[{"PlotLabel", "\[Rule]", " ", "\"\<M\>\""}]}], "]"}], ",", 
    RowBox[{"ListPlot", "[", 
     RowBox[{"enlist", ",", 
      RowBox[{"PlotLabel", "\[Rule]", " ", "\"\<Energy\>\""}]}], "]"}]}], 
   "}"}], "//", "Dynamic"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Mean", "[", 
   RowBox[{"magnlist", "//", "Abs"}], "]"}], "//", "Dynamic"}]}], "Input",
 CellChangeTimes->{{3.7469628416690197`*^9, 3.7469628465755043`*^9}, {
   3.7469628962971897`*^9, 3.746962898687656*^9}, 3.746963000563311*^9, {
   3.746963167976309*^9, 3.7469632625766287`*^9}, {3.749904373554007*^9, 
   3.7499043892743607`*^9}},ExpressionUUID->"8713bcd7-095f-419b-80bf-\
312543bafac3"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cluster update", "Subsection",
 CellChangeTimes->{{3.746515817793416*^9, 
  3.746515818977425*^9}},ExpressionUUID->"91e718fb-dc0a-4496-8cbf-\
859a29f1385b"],

Cell["\<\
You might have noticed (probably not...) that your local update is slow to \
forget about the previous configuration when \[Beta] is close to its critical \
value or higher. There is a way around this problem by considering larger \
updates. Consider any lattice configuration and pick site i. Site i is the \
first site included in your cluster. You increase the cluster as follows: For \
all the sites in the cluster, consider the nearest neighbours to it. If they \
have the same spin, connect the two sites with probability 1-Exp[-2 \[Beta] \
J]. If they don\[CloseCurlyQuote]t have the same spin, ignore the site. Note \
that a site can be joined to the cluster from more than one neighbour so you \
can try to connect it many times. Once there are no more sites to add to the \
cluster, you flip the spin in all the sites of the cluster. And you start all \
over again. (Shortly: clusters are connected regions whose sites all have \
their spins aligned and new sites are connected with probability 1-exp(-2 \
\[Beta] J))

These are known as Fortuin-Kasteleyn clusters and the update method is known \
as the Wolff algorithm. These are much more efficient in sampling new new \
independent lattice configurations than local updates when \[Beta] increases. \
In our implementation, h=0 implicitly. There are ways to add h to the cluster \
method but it is out of the scope of this project (see e.g. ghost spins).\
\>", "Text",
 CellChangeTimes->{{3.746964039131575*^9, 3.746964077864079*^9}, {
  3.746964111521885*^9, 3.746964292098406*^9}, {3.7469643590638037`*^9, 
  3.746964623424906*^9}, {3.746964770108532*^9, 
  3.746964811590372*^9}},ExpressionUUID->"c8105c01-a104-4269-ad3b-\
eb7f890c6bac"],

Cell[CellGroupData[{

Cell["\<\
8. Construct a second lattice (array) for keeping track of the sites already \
visited by the cluster expansion algorithm.\
\>", "Subsubsection",
 CellChangeTimes->{{3.746963993534148*^9, 3.746964036598456*^9}, {
  3.7469646334466476`*^9, 
  3.746964671830331*^9}},ExpressionUUID->"b0cec9f1-89ed-4040-890e-\
f5dd4042cd32"],

Cell[BoxData[
 RowBox[{
  RowBox[{"latticei", " ", "=", " ", 
   RowBox[{"ConstantArray", "[", 
    RowBox[{"0", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.7469646830192413`*^9, 
  3.746964700362541*^9}},ExpressionUUID->"23bd8614-bf78-4b28-8028-\
b4c98d6a6d07"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
9. Construct an algorithm for growing a FK cluster and then flip the spins\
\>", "Subsubsection",
 CellChangeTimes->{{3.746964709518553*^9, 
  3.746964732917432*^9}},ExpressionUUID->"c64ec354-18c2-4737-97f9-\
e39eb85732b0"],

Cell["\<\
First some auxiliary functions for getting accessing array values.\
\>", "Text",
 CellChangeTimes->{{3.746965162433708*^9, 
  3.746965177494822*^9}},ExpressionUUID->"c627282a-b222-4d5b-be64-\
a3ed2a0f12f0"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"spin", "[", "site_", "]"}], ":=", 
   RowBox[{"lattice0", "[", 
    RowBox[{"[", 
     RowBox[{
      RowBox[{"site", "[", 
       RowBox[{"[", "1", "]"}], "]"}], ",", 
      RowBox[{"site", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"index", "[", "site_", "]"}], ":=", 
   RowBox[{"latticei", "[", 
    RowBox[{"[", 
     RowBox[{
      RowBox[{"site", "[", 
       RowBox[{"[", "1", "]"}], "]"}], ",", 
      RowBox[{"site", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.7455866519020653`*^9, 3.745586745323392*^9}, {
  3.745587090449614*^9, 
  3.745587107751521*^9}},ExpressionUUID->"e5604195-2bde-49c3-8e7a-\
160cc19fdf30"],

Cell["\<\
We first make a list of sites in the cluster and start going through the \
suitable nearest neighbours of the sites in this list ignoring those that \
have already been added to the list (indicated by latticei). Translation \
note: \[OpenCurlyDoubleQuote]saitti\[CloseCurlyDoubleQuote] = \
\[OpenCurlyDoubleQuote]site\[CloseCurlyDoubleQuote].\
\>", "Text",
 CellChangeTimes->{{3.7469651904863997`*^9, 3.7469651990054407`*^9}, {
  3.746965276267641*^9, 3.746965316524634*^9}, {3.7469653606842213`*^9, 
  3.7469653728835173`*^9}, {3.749904567495035*^9, 
  3.749904576639027*^9}},ExpressionUUID->"2109e4e4-050a-4dbc-bd58-\
5349355b84f0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"constructcluster", "[", 
   RowBox[{"\[Beta]_", ",", "J_", ",", "site_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"prob", "=", 
       RowBox[{
        RowBox[{"1", "-", 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"-", "2"}], "\[Beta]", " ", "J"}], "]"}]}], "//", "N"}]}], 
      ",", 
      RowBox[{"clusterlist", "=", 
       RowBox[{"{", "site", "}"}]}]}], "}"}], ",", 
    RowBox[{"(*", 
     RowBox[{
     "intial", " ", "site", " ", "is", " ", "in", " ", "the", " ", 
      "cluster"}], "*)"}], 
    RowBox[{
     RowBox[{"latticei", "=", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{"0", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
           RowBox[{"[", "1", "]"}], "]"}], ",", 
          RowBox[{
           RowBox[{"Dimensions", "[", "lattice0", "]"}], "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], "}"}]}], "]"}]}], ";", 
     RowBox[{
      RowBox[{"latticei", "[", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"site", "[", 
          RowBox[{"[", "1", "]"}], "]"}], ",", 
         RowBox[{"site", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "]"}], "=", "1"}], ";", 
     RowBox[{"(*", 
      RowBox[{
      "Label", " ", "the", " ", "first", " ", "site", " ", "as", " ", 
       "included", " ", "in", " ", "the", " ", "lattice"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
      "Start", " ", "systematically", " ", "going", " ", "through", " ", 
       "all", " ", "the", " ", "sites", " ", "in", " ", "the", " ", "cluster",
        " ", "while", " ", "it", " ", "is", " ", "growsing"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "clusterlist", "]"}]}], ",", 
       RowBox[{"i", "=", 
        RowBox[{"i", "+", "1"}]}], ",", 
       RowBox[{
        RowBox[{"saitti", "=", 
         RowBox[{"clusterlist", "[", 
          RowBox[{"[", "i", "]"}], "]"}]}], ";", 
        RowBox[{"aux", "=", 
         RowBox[{"siteup", "[", "saitti", "]"}]}], ";", " ", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
            "We", " ", "save", " ", "some", " ", "time", " ", "by", " ", 
             "making", " ", "copy"}], "-", 
            RowBox[{"pasteable", " ", "code"}]}], "..."}], " ", "aux", " ", 
          "is", " ", "auxiliary", " ", "variable", " ", "for", " ", 
          "considering", " ", "the", " ", "next", " ", "site", " ", "to", " ",
           "be", " ", "added", " ", "to", " ", "the", " ", "cluster"}], 
         "*)"}], 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"spin", "[", "aux", "]"}], "\[Equal]", 
            RowBox[{"spin", "[", "saitti", "]"}]}], "&&", 
           RowBox[{
            RowBox[{"RandomReal", "[", 
             RowBox[{"{", 
              RowBox[{"0", ",", "1"}], "}"}], "]"}], "\[LessEqual]", " ", 
            "prob"}], "&&", 
           RowBox[{
            RowBox[{"index", "[", "aux", "]"}], "\[Equal]", "0"}]}], ",", 
          RowBox[{
           RowBox[{"clusterlist", "=", 
            RowBox[{"Append", "[", 
             RowBox[{"clusterlist", ",", "aux"}], "]"}]}], ";", 
           RowBox[{
            RowBox[{"latticei", "[", 
             RowBox[{"[", 
              RowBox[{
               RowBox[{"aux", "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", 
               RowBox[{"aux", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "]"}], "=", "1"}]}], 
          ",", 
          RowBox[{"1", ";"}]}], "]"}], ";", " ", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{
          "Check", " ", "if", " ", "spins", " ", "match", " ", "and", " ", 
           "if", " ", "the", " ", "site", " ", "is", " ", "not", " ", 
           "already", " ", "in", " ", "the", " ", 
           RowBox[{"cluster", ".", " ", "Then"}], " ", "determine", " ", 
           "whether", " ", "or", " ", "not", " ", "it", " ", "will", " ", 
           "be", " ", "included", " ", 
           RowBox[{
            RowBox[{"(", "random", ")"}], ".", " ", "If"}], " ", "yes"}], ",",
           " ", 
          RowBox[{
          "mark", " ", "the", " ", "site", " ", "as", " ", "included"}]}], 
         "*)"}], 
        RowBox[{"aux", " ", "=", " ", 
         RowBox[{"sitedown", "[", "saitti", "]"}]}], ";", " ", 
        RowBox[{"(*", 
         RowBox[{"next", " ", "direction"}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"spin", "[", "aux", "]"}], "\[Equal]", 
            RowBox[{"spin", "[", "saitti", "]"}]}], "&&", 
           RowBox[{
            RowBox[{"RandomReal", "[", 
             RowBox[{"{", 
              RowBox[{"0", ",", "1"}], "}"}], "]"}], "<=", "prob"}], "&&", 
           RowBox[{
            RowBox[{"index", "[", "aux", "]"}], "\[Equal]", "0"}]}], ",", 
          RowBox[{
           RowBox[{"clusterlist", "=", 
            RowBox[{"Append", "[", 
             RowBox[{"clusterlist", ",", "aux"}], "]"}]}], ";", 
           RowBox[{
            RowBox[{"latticei", "[", 
             RowBox[{"[", 
              RowBox[{
               RowBox[{"aux", "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", 
               RowBox[{"aux", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "]"}], "=", "1"}]}], 
          ",", 
          RowBox[{"1", ";"}]}], "]"}], ";", 
        RowBox[{"aux", "=", 
         RowBox[{"siteleft", "[", "saitti", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"spin", "[", "aux", "]"}], "\[Equal]", 
            RowBox[{"spin", "[", "saitti", "]"}]}], "&&", 
           RowBox[{
            RowBox[{"RandomReal", "[", 
             RowBox[{"{", 
              RowBox[{"0", ",", "1"}], "}"}], "]"}], "\[LessEqual]", " ", 
            "prob"}], "&&", 
           RowBox[{
            RowBox[{"index", "[", "aux", "]"}], "\[Equal]", "0"}]}], ",", 
          RowBox[{
           RowBox[{"clusterlist", "=", 
            RowBox[{"Append", "[", 
             RowBox[{"clusterlist", ",", "aux"}], "]"}]}], ";", 
           RowBox[{
            RowBox[{"latticei", "[", 
             RowBox[{"[", 
              RowBox[{
               RowBox[{"aux", "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", 
               RowBox[{"aux", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "]"}], "=", "1"}]}], 
          ",", 
          RowBox[{"1", ";"}]}], "]"}], ";", 
        RowBox[{"aux", "=", 
         RowBox[{"siteright", "[", "saitti", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"spin", "[", "aux", "]"}], "\[Equal]", 
            RowBox[{"spin", "[", "saitti", "]"}]}], "&&", 
           RowBox[{
            RowBox[{"RandomReal", "[", 
             RowBox[{"{", 
              RowBox[{"0", ",", "1"}], "}"}], "]"}], "\[LessEqual]", " ", 
            "prob"}], "&&", 
           RowBox[{
            RowBox[{"index", "[", "aux", "]"}], "\[Equal]", "0"}]}], ",", 
          RowBox[{
           RowBox[{"clusterlist", "=", 
            RowBox[{"Append", "[", 
             RowBox[{"clusterlist", ",", "aux"}], "]"}]}], ";", 
           RowBox[{
            RowBox[{"latticei", "[", 
             RowBox[{"[", 
              RowBox[{
               RowBox[{"aux", "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", 
               RowBox[{"aux", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "]"}], "=", "1"}]}], 
          ",", 
          RowBox[{"1", ";"}]}], "]"}], ";"}]}], "]"}], ";", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", " ", 
        RowBox[{"Length", "[", "clusterlist", "]"}]}], ",", 
       RowBox[{"i", "=", 
        RowBox[{"i", "+", "1"}]}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"lattice0", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"clusterlist", "[", 
             RowBox[{"[", 
              RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", 
            RowBox[{"clusterlist", "[", 
             RowBox[{"[", 
              RowBox[{"i", ",", "2"}], "]"}], "]"}]}], "]"}], "]"}], "=", 
         RowBox[{"-", 
          RowBox[{"lattice0", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"clusterlist", "[", 
              RowBox[{"[", 
               RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", 
             RowBox[{"clusterlist", "[", 
              RowBox[{"[", 
               RowBox[{"i", ",", "2"}], "]"}], "]"}]}], "]"}], "]"}]}]}], 
        ";"}]}], "]"}]}]}], 
   RowBox[{"(*", 
    RowBox[{
    "flip", " ", "all", " ", "the", " ", "sites", " ", "in", " ", "the", " ", 
     "cluster"}], "*)"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.7455857222730703`*^9, 3.745585725184668*^9}, {
   3.745585783976439*^9, 3.7455858277600927`*^9}, {3.745585874608699*^9, 
   3.7455859306473513`*^9}, {3.745586037096511*^9, 3.745586054445654*^9}, {
   3.745586758562479*^9, 3.745586937985682*^9}, {3.745587013356345*^9, 
   3.7455870852045727`*^9}, {3.745587120528811*^9, 3.745587366702725*^9}, {
   3.74558740672954*^9, 3.745587435761086*^9}, {3.7455874668822308`*^9, 
   3.745587470728393*^9}, {3.74558750128688*^9, 3.745587529906211*^9}, {
   3.7455875766367683`*^9, 3.745587580105497*^9}, {3.7455876659030457`*^9, 
   3.7455876876781073`*^9}, {3.745587732516633*^9, 3.745587856288046*^9}, {
   3.74558802166284*^9, 3.745588023511744*^9}, {3.7455880640586777`*^9, 
   3.745588068281024*^9}, {3.745588119996749*^9, 3.745588201883698*^9}, {
   3.74558880216615*^9, 3.745588836543344*^9}, {3.745588873166613*^9, 
   3.745588874923074*^9}, {3.745588917101214*^9, 3.745588944916081*^9}, {
   3.745589149164716*^9, 3.745589162868018*^9}, {3.7455892018117323`*^9, 
   3.7455892174670753`*^9}, 3.745589276295223*^9, {3.745589776244783*^9, 
   3.745589784202711*^9}, 3.7455898924858227`*^9, {3.74559039342907*^9, 
   3.745590394505683*^9}, {3.745590517402265*^9, 3.745590538315933*^9}, {
   3.7455910480746517`*^9, 3.745591115634335*^9}, {3.745591209615975*^9, 
   3.745591296209735*^9}, {3.74559133736709*^9, 3.745591364004328*^9}, 
   3.745591400441073*^9, 3.746965515814198*^9, {3.7499044998722553`*^9, 
   3.7499045579138308`*^9}, {3.749904594603551*^9, 3.7499046012194843`*^9}, {
   3.749904633179826*^9, 3.7499047022135563`*^9}, {3.74990480979239*^9, 
   3.749904850537984*^9}},ExpressionUUID->"eb24c854-6329-4c8a-b624-\
cf0661a12a6d"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
10. Same as problem 7 but use your new cluster method. This time, h=0 in our \
implementation. Analysis is skipped in these solutions\
\>", "Subsubsection",
 CellChangeTimes->{{3.746964742882045*^9, 3.74696476554834*^9}, {
  3.749904967505118*^9, 
  3.749904974616733*^9}},ExpressionUUID->"878aa0cc-d46a-4bd7-bc1b-\
714b00d659e4"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"magn", "=", 
   RowBox[{"{", "}"}]}], ";", 
  RowBox[{"energy", "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"J", "=", "1.0"}], ";", " ", 
  RowBox[{"h", " ", "=", " ", "0"}], ";", " ", 
  RowBox[{"\[Beta]", "=", "0.44"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"constructcluster", "[", 
     RowBox[{"\[Beta]", ",", "J", ",", 
      RowBox[{"randomsite", "[", "]"}]}], "]"}], ";", 
    RowBox[{"magn", "=", 
     RowBox[{"Append", "[", 
      RowBox[{"magn", ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Total", "[", 
          RowBox[{"lattice0", ",", "2"}], "]"}], "/", 
         RowBox[{"Times", "@@", 
          RowBox[{"Dimensions", "[", "lattice0", "]"}]}]}], "//", "N"}]}], 
      "]"}]}], ";", 
    RowBox[{"energy", "=", 
     RowBox[{"Append", "[", 
      RowBox[{"energy", ",", 
       RowBox[{
        RowBox[{"Ham", "[", 
         RowBox[{"J", ",", "h"}], "]"}], "//", "N"}]}], "]"}]}], ";"}], ",", 
   RowBox[{"{", "500", "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.745572013990861*^9, 3.745572020950851*^9}, {
   3.745572155224204*^9, 3.7455721917610903`*^9}, {3.74557223240947*^9, 
   3.745572233032812*^9}, {3.745572281417892*^9, 3.745572281849265*^9}, {
   3.745572486804648*^9, 3.7455725317810793`*^9}, {3.745572576094425*^9, 
   3.745572603886117*^9}, {3.745578049959756*^9, 3.7455780510728407`*^9}, {
   3.7455781177604322`*^9, 3.7455781268400583`*^9}, 3.745585372258*^9, {
   3.7455855494581823`*^9, 3.745585550231682*^9}, 3.7455855898648987`*^9, 
   3.745591513454175*^9, 3.745591638079103*^9, {3.7455917046371307`*^9, 
   3.745591704929134*^9}, {3.7455920545154753`*^9, 3.745592055115387*^9}, {
   3.745592476447104*^9, 3.745592479712784*^9}, {3.746964932448751*^9, 
   3.746964967562662*^9}, {3.7469650171863422`*^9, 3.746965017268388*^9}, {
   3.746965059367578*^9, 
   3.7469650803277683`*^9}},ExpressionUUID->"e5d1cb91-3baf-48b0-bdc0-\
6e9ea7886705"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Dynamic", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"MatrixPlot", "[", "lattice0", "]"}], ",", 
    RowBox[{"ListPlot", "[", 
     RowBox[{"magn", ",", 
      RowBox[{"PlotLabel", "\[Rule]", " ", "\"\<M\>\""}]}], "]"}], ",", 
    RowBox[{"ListPlot", "[", 
     RowBox[{"energy", ",", 
      RowBox[{"PlotLabel", "\[Rule]", " ", "\"\<Energy\>\""}]}], "]"}]}], 
   "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Dynamic", "[", 
  RowBox[{"Mean", "[", 
   RowBox[{"Abs", "[", "magn", "]"}], "]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.745571592981003*^9, 3.7455716127002068`*^9}, {
   3.745572093528287*^9, 3.74557213858391*^9}, {3.745572250106791*^9, 
   3.7455722526332827`*^9}, {3.745572610150866*^9, 3.745572614310054*^9}, 
   3.7455907472274113`*^9, {3.7455913529105186`*^9, 3.745591356093834*^9}, {
   3.746516233448245*^9, 3.746516249191719*^9}, {3.7465163044655857`*^9, 
   3.7465163073230753`*^9}, {3.746964837298972*^9, 3.74696486625657*^9}, {
   3.746964977357739*^9, 3.7469649792396593`*^9}, {3.7499048692141113`*^9, 
   3.749904881849057*^9}},ExpressionUUID->"dfa61e1e-b9d0-4e69-ae5d-\
a94738573f05"],

Cell[BoxData[
 DynamicBox[ToBoxes[{
    MatrixPlot[$CellContext`lattice0], 
    ListPlot[$CellContext`magn, PlotLabel -> "M"], 
    ListPlot[$CellContext`energy, PlotLabel -> "Energy"]}, StandardForm],
  ImageSizeCache->{582., {86., 90.}}]], "Output",
 CellChangeTimes->{3.746964887507144*^9, 3.746964981123283*^9, 
  3.74990492204856*^9},ExpressionUUID->"e85b3807-057b-4546-992f-a281c04aece0"],

Cell[BoxData[
 DynamicBox[ToBoxes[
   Mean[
    Abs[$CellContext`magn]], StandardForm],
  ImageSizeCache->{74., {0., 9.}}]], "Output",
 CellChangeTimes->{3.746964887507144*^9, 3.746964981123283*^9, 
  3.749904922328549*^9},ExpressionUUID->"f0321ba5-b7a9-4a27-9849-\
99a10ed3cf35"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1147, 716},
WindowMargins->{{97, Automatic}, {Automatic, 70}},
ShowSelection->True,
FrontEndVersion->"11.1 for Linux x86 (64-bit) (April 18, 2017)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 232, 4, 93, "Title", "ExpressionUUID" -> \
"9b3c0e5a-c573-4b61-a303-a50bac06aac4"],
Cell[815, 28, 3683, 79, 300, "Text", "ExpressionUUID" -> \
"b7040563-df6a-43ce-8104-d86234ae13d2"],
Cell[CellGroupData[{
Cell[4523, 111, 183, 3, 72, "Section", "ExpressionUUID" -> \
"99eafa64-8388-4295-ba90-f9acbafb29be"],
Cell[CellGroupData[{
Cell[4731, 118, 168, 3, 44, "Subsection", "ExpressionUUID" -> \
"18ec621e-585d-4060-8e7d-6d2be6ea8cc1"],
Cell[CellGroupData[{
Cell[4924, 125, 210, 4, 35, "Subsubsection", "ExpressionUUID" -> \
"61df1d94-7c8f-4b08-88b5-80d41d496fba"],
Cell[5137, 131, 182, 3, 32, "Input", "ExpressionUUID" -> \
"93daf99f-b2fa-4394-ad5f-7aa3fe99cec2"]
}, Closed]],
Cell[CellGroupData[{
Cell[5356, 139, 439, 8, 53, "Subsubsection", "ExpressionUUID" -> \
"2c104146-3087-412a-aea7-a19b313f60c6"],
Cell[5798, 149, 1227, 28, 57, "Input", "ExpressionUUID" -> \
"9c39febb-a112-4f1e-b2cc-894fef987861"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7062, 182, 324, 7, 35, "Subsubsection", "ExpressionUUID" -> \
"3f441c09-93ab-43ad-8733-54f165d749b9"],
Cell[7389, 191, 2586, 79, 126, "Input", "ExpressionUUID" -> \
"49254c77-89c5-48d5-9bcb-cb7a7a4eb1a7"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10012, 275, 230, 5, 35, "Subsubsection", "ExpressionUUID" -> \
"fd907b3e-1a13-4c41-9ed7-c678a87a4c60"],
Cell[10245, 282, 3162, 83, 147, "Input", "ExpressionUUID" -> \
"4096833e-5303-4bd0-a79e-ed4ad8913ede"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13444, 370, 222, 5, 35, "Subsubsection", "ExpressionUUID" -> \
"f2e72b12-f57c-489a-bdd1-2f86c0748720"],
Cell[13669, 377, 619, 16, 34, "Input", "ExpressionUUID" -> \
"7006cbf9-205b-4cf2-b031-c994d43de44c"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[14337, 399, 160, 3, 45, "Subsection", "ExpressionUUID" -> \
"7f602e24-833f-440f-b2f8-757f88ead821"],
Cell[14500, 404, 1052, 16, 91, "Text", "ExpressionUUID" -> \
"55d33a9c-336d-4330-83f7-136351ebe0ba"],
Cell[15555, 422, 397, 8, 51, "Text", "ExpressionUUID" -> \
"0cecc518-0fae-4b80-852d-1d066b1bd577"],
Cell[CellGroupData[{
Cell[15977, 434, 347, 7, 35, "Subsubsection", "ExpressionUUID" -> \
"83454930-bae6-4357-809c-5b3a00a914c9"],
Cell[16327, 443, 1030, 28, 52, "Text", "ExpressionUUID" -> \
"c34ec28b-1878-4ea6-9bdd-d8ecaedadb17"],
Cell[17360, 473, 5667, 153, 307, "Input", "ExpressionUUID" -> \
"56115e78-f935-427c-8548-79261f708911"]
}, Open  ]],
Cell[CellGroupData[{
Cell[23064, 631, 250, 4, 35, "Subsubsection", "ExpressionUUID" -> \
"8f4a53c4-3da9-4248-ab55-eba7508aa5e0"],
Cell[23317, 637, 1422, 31, 56, "Input", "ExpressionUUID" -> \
"56115e78-f935-427c-8548-79261f708911"]
}, Open  ]],
Cell[CellGroupData[{
Cell[24776, 673, 1682, 39, 152, "Subsubsection", "ExpressionUUID" -> \
"8f17aa57-d4a5-45da-8acb-6c1d68fd666c"],
Cell[CellGroupData[{
Cell[26483, 716, 1182, 29, 103, "Input", "ExpressionUUID" -> \
"d8b87073-f51b-4329-81f0-7694664750f3"],
Cell[27668, 747, 279, 5, 32, "Output", "ExpressionUUID" -> \
"d6a1620d-bdbd-402a-8cd4-c79fde3f63e0"]
}, Open  ]],
Cell[27962, 755, 843, 19, 56, "Input", "ExpressionUUID" -> \
"8713bcd7-095f-419b-80bf-312543bafac3"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[28854, 780, 162, 3, 45, "Subsection", "ExpressionUUID" -> \
"91e718fb-dc0a-4496-8cbf-859a29f1385b"],
Cell[29019, 785, 1713, 26, 230, "Text", "ExpressionUUID" -> \
"c8105c01-a104-4269-ad3b-eb7f890c6bac"],
Cell[CellGroupData[{
Cell[30757, 815, 332, 7, 35, "Subsubsection", "ExpressionUUID" -> \
"b0cec9f1-89ed-4040-890e-f5dd4042cd32"],
Cell[31092, 824, 555, 15, 34, "Input", "ExpressionUUID" -> \
"23bd8614-bf78-4b28-8028-b4c98d6a6d07"]
}, Open  ]],
Cell[CellGroupData[{
Cell[31684, 844, 233, 5, 35, "Subsubsection", "ExpressionUUID" -> \
"c64ec354-18c2-4737-97f9-e39eb85732b0"],
Cell[31920, 851, 216, 5, 31, "Text", "ExpressionUUID" -> \
"c627282a-b222-4d5b-be64-a3ed2a0f12f0"],
Cell[32139, 858, 818, 25, 57, "Input", "ExpressionUUID" -> \
"e5604195-2bde-49c3-8e7a-160cc19fdf30"],
Cell[32960, 885, 644, 11, 51, "Text", "ExpressionUUID" -> \
"2109e4e4-050a-4dbc-bd58-5349355b84f0"],
Cell[33607, 898, 10928, 265, 535, "Input", "ExpressionUUID" -> \
"eb24c854-6329-4c8a-b624-cf0661a12a6d"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44572, 1168, 340, 7, 59, "Subsubsection", "ExpressionUUID" -> \
"878aa0cc-d46a-4bd7-bc1b-714b00d659e4"],
Cell[44915, 1177, 2017, 46, 126, "Input", "ExpressionUUID" -> \
"e5d1cb91-3baf-48b0-bdc0-6e9ea7886705"],
Cell[CellGroupData[{
Cell[46957, 1227, 1143, 23, 56, "Input", "ExpressionUUID" -> \
"dfa61e1e-b9d0-4e69-ae5d-a94738573f05"],
Cell[48103, 1252, 394, 7, 193, "Output", "ExpressionUUID" -> \
"e85b3807-057b-4546-992f-a281c04aece0"],
Cell[48500, 1261, 280, 7, 32, "Output", "ExpressionUUID" -> \
"f0321ba5-b7a9-4a27-9849-99a10ed3cf35"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

